from ezetl.data_models import DataModel
from ezetl.utils.common_utils import trans_rule_value, gen_json_response, format_date
from ezetl.libs.prometheus import PrometheusClient


class BasePromModel(DataModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        self.conn_conf = self._source['conn_conf']
        self.model_conf = self._model.get('model_conf', {})
        self.url = self.conn_conf.get('url')
        self.auth_types = self.model_conf.get('auth_type', '').split(',')
        self._client = PrometheusClient(**{"url": self.url, 'disable_ssl': True})

    def connect(self):
        '''
        连通性测试
        '''
        try:
            self._client.conn_test()
            return True, '连接成功'
        except Exception as e:
            return False, str(e)[:100]


class PromMetricModel(BasePromModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        self.metric = self.model_conf.get('name')

    def connect(self):
        '''
        连通性测试
        '''
        try:
            all_metrics = self._client.get_all_metrics()
            if self.metric in all_metrics:
                return True, '连接成功'
            else:
                return False, '未找到该指标'
        except Exception as e:
            return False, str(e)[:100]

    def get_res_fields(self):
        '''
        获取字段列表
        '''
        res_fields = []
        try:
            pass
        except Exception as e:
            print(e)
        return res_fields

    def get_search_type_list(self):
        '''
        获取可用高级查询类型
        '''
        return []

    def get_extract_rules(self):
        '''
        获取可筛选项
        :return:
        '''
        rules = [{
            'name': '开始时间',
            'value': 'start_time',
            "default": ''
        }, {
            'name': '结束时间',
            'value': 'end_time',
            "default": ''
        }, {
            'name': '步长',
            'value': 'step',
            "default": ''
        }]
        return rules

    def gen_extract_rules(self):
        '''
        解析筛选规则
        :return:
        '''
        try:
            query_dict = {}
            for i in self.extract_rules:
                field = i.get('field')
                rule = i.get('rule')
                value = i.get('value')
                value = trans_rule_value(value)
                if field and value:
                    if rule == 'start_time':
                        start_time = format_date(value, res_type='timestamp')
                        if start_time is not None:
                            query_dict['start_time'] = start_time
                    if rule == 'end_time':
                        end_time = format_date(value, res_type='timestamp')
                        if end_time is not None:
                            query_dict['end_time'] = end_time
                    if rule == 'step':
                        try:
                            query_dict['step'] = float(value)
                        except Exception as e:
                            print(e)
            data_li = self._client.query(self.metric, **query_dict)
            return True, data_li
        except Exception as e:
            return False, str(e)[:500]

    def read_page(self, page=1, pagesize=20):
        '''
        分页读取数据
        :param page:
        :param pagesize:
        :return:
        '''
        flag, res_data = self.gen_extract_rules()
        if not flag:
            return False, res_data
        total = len(res_data)
        data_li = res_data[(page - 1) * pagesize:page * pagesize]
        res_data = {
            'records': data_li,
            'total': total
        }
        return True, gen_json_response(data=res_data)

    def read_batch(self):
        '''
        生成器分批读取数据
        :return:
        '''
        flag, res_data = self.gen_extract_rules()
        if not flag:
            yield False, res_data
        total = len(res_data)
        pagesize = self._extract_info.get('batch_size', 1000)
        total_pages = total // pagesize + 1
        n = 0
        while n < total_pages:
            page = n + 1
            n += 1
            result = {
                'records': res_data[(page - 1) * pagesize:page * pagesize],
                'total': total
            }
            yield True, gen_json_response(result)


class PromQlModel(BasePromModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        promql = self.model_conf.get('promql')
        self.promql = promql
        self.default_promql = promql

    def get_res_fields(self):
        '''
        获取字段列表
        '''
        res_fields = []
        try:
            pass
        except Exception as e:
            print(e)
        return res_fields

    def get_search_type_list(self):
        '''
        获取可用高级查询类型
        '''
        return [{
            'name': 'promql',
            'value': 'promql',
            "default": self.promql
        }]

    def get_extract_rules(self):
        '''
        获取可筛选项
        :return:
        '''
        rules = [{
            'name': '开始时间',
            'value': 'start_time',
            "default": ''
        }, {
            'name': '结束时间',
            'value': 'end_time',
            "default": ''
        }, {
            'name': '步长',
            'value': 'step',
            "default": ''
        }]
        return rules

    def gen_extract_rules(self):
        '''
        解析筛选规则
        :return:
        '''
        try:
            rules = [i for i in self.extract_rules if i['field'] == 'search_text' and i['rule'] == 'promql' and i['value']]
            if rules != []:
                self.promql = rules[0].get('value')
            if 'custom_sql' not in self.auth_types and self.promql != self.default_promql:
                return False, '无修改promql权限'
            query_dict = {}
            for i in self.extract_rules:
                field = i.get('field')
                rule = i.get('rule')
                value = i.get('value')
                value = trans_rule_value(value)
                if field and value:
                    if rule == 'start_time':
                        start_time = format_date(value, res_type='timestamp')
                        if start_time is not None:
                            query_dict['start_time'] = start_time
                    if rule == 'end_time':
                        end_time = format_date(value, res_type='timestamp')
                        if end_time is not None:
                            query_dict['end_time'] = end_time
                    if rule == 'step':
                        try:
                            query_dict['step'] = float(value)
                        except Exception as e:
                            print(e)
            data_li = self._client.query(self.promql, **query_dict)
            return True, data_li
        except Exception as e:
            return False, str(e)[:500]

    def read_page(self, page=1, pagesize=20):
        '''
        分页读取数据
        :param page:
        :param pagesize:
        :return:
        '''
        flag, res_data = self.gen_extract_rules()
        if not flag:
            return False, res_data
        total = len(res_data)
        data_li = res_data[(page - 1) * pagesize:page * pagesize]
        res_data = {
            'records': data_li,
            'total': total
        }
        return True, gen_json_response(data=res_data)

    def read_batch(self):
        '''
        生成器分批读取数据
        :return:
        '''
        flag, res_data = self.gen_extract_rules()
        if not flag:
            yield False, res_data
        total = len(res_data)
        pagesize = self._extract_info.get('batch_size', 1000)
        total_pages = total // pagesize + 1
        n = 0
        while n < total_pages:
            page = n + 1
            n += 1
            result = {
                'records': res_data[(page - 1) * pagesize:page * pagesize],
                'total': total
            }
            yield True, gen_json_response(result)


from ezetl.data_models import DataModel


class HdfsModel(DataModel):

    def __init__(self, model_info):
        super().__init__(model_info)


import hdfs3

# 创建 HDFS 客户端连接
client = hdfs3.HDFileSystem(host='127.0.0.1', port=50070, user='hadoop')
with client.open('/path/to/file.txt', 'rb') as f:
    data = f.read()

# 将数据写入 HDFS
with client.open('/path/to/output.txt', 'wb') as f:
    f.write(data)


from ezetl.data_models.base_db_table import BaseDBTableModel
from ezetl.data_models.base_db_sql import BaseDBSqlModel
from sqlalchemy import Column, String, MetaData, Table


def gen_string_column():
    obj = Column(String)
    return obj


class HiveTableModel(BaseDBTableModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        self.column_gen_map = {
            'TEXT': gen_string_column,
            'String': gen_string_column
        }

    def create(self, field_arr=[]):
        '''
        创建表
        '''
        if 'create' not in self.auth_types:
            return False, '无创建权限'
        try:
            metadata = MetaData()
            metadata.reflect(bind=self.db_engine)
            if self.table_name not in metadata.tables.keys():
                # 定义表格结构
                _table = Table(self.table_name, metadata)
                for i in field_arr:
                    if i['type'] in self.column_gen_map:
                        column = self.column_gen_map[i['type']](i)
                    else:
                        column = self.genColumn(i)
                    column.name = i['field_value']
                    _table.append_column(column)
                # 执行建表操作
                metadata.create_all(self.db_engine)
            self.db_engine.dispose()
            # 重新获取模型
            self.get_table_model()
            return True, '创建成功'
        except Exception as e:
            return False, str(e)


class HiveSqlModel(BaseDBSqlModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        model_conf = self._model.get('model_conf', {})
        self.sql = model_conf.get('sql', 'SHOW TABLES')
        self.default_sql = self.sql


from ezetl.data_models.base_db_table import BaseDBTableModel
from ezetl.data_models.base_db_sql import BaseDBSqlModel
from sqlalchemy import Column, TEXT


def gen_string_column(i):
    if 'nullable' in i.keys():
        i['nullable'] = i['nullable'] == 1
    else:
        i['nullable'] = True
    obj = Column(TEXT, nullable=i['nullable'])
    return obj


class PgsqlTableModel(BaseDBTableModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        self.column_gen_map = {
            'String': gen_string_column
        }


class PgsqlSqlModel(BaseDBSqlModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        model_conf = self._model.get('model_conf', {})
        self.sql = model_conf.get('sql', 'select * from pg_tables')
        self.default_sql = self.sql


from ezetl.data_models import DataModel
from ezetl.utils.common_utils import trans_rule_value, gen_json_response, parse_json, request_url


class BaseHttpModel(DataModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        self.conn_conf = self._source['conn_conf']
        self.model_conf = self._model.get('model_conf', {})
        self.url = self.conn_conf.get('url')
        self.method = self.conn_conf.get('method')
        self.headers = parse_json(self.conn_conf.get('headers', {}))
        try:
            self.timeout = int(self.conn_conf.get('timeout'), 60)
        except Exception as e:
            print(e)
            self.timeout = 60
        self.req_body = parse_json(self.conn_conf.get('req_body'), {})
        self.req_params = {}
        self.auth_types = self.model_conf.get('auth_type', '').split(',')

    def get_search_type_list(self):
        '''
        获取可用高级查询类型
        '''
        return [{
            'name': '请求参数',
            'value': 'req_body',
            "default": str(self.req_body)
        }]

    def connect(self):
        '''
        连通性测试
        '''
        try:
            body_rules = [i for i in self.extract_rules if
                        i['field'] == 'search_text' and i['rule'] == 'req_body' and i['value']]
            if body_rules != []:
                req_body = parse_json(body_rules[0]['value'])
                if req_body is not None:
                    if self.method == 'get':
                        self.req_params = req_body
                    else:
                        self.req_body = req_body
            self.response = request_url(method=self.method, url=self.url, headers=self.headers, params=self.req_params, data=self.req_body, timeout=self.timeout)
            return True, '连接成功'
        except Exception as e:
            return False, str(e)[:100]


class HttpApiModel(BaseHttpModel):

    def __init__(self, model_info):
        super().__init__(model_info)

    def read_page(self, page=1, pagesize=20):
        '''
        分页读取数据
        :param page:
        :param pagesize:
        :return:
        '''
        flag, err = self.connect()
        if not flag:
            return False, err
        try:
            result = parse_json(self.response.text)
            if isinstance(result, list):
                total = len(result)
                data_li = result
            else:
                total = 1
                data_li = [result]
            res_data = {
                'records': data_li,
                'total': total
            }
            return True, gen_json_response(data=res_data)
        except Exception as e:
            return False, str(e)[:200]

    def read_batch(self):
        '''
        生成器分批读取数据
        :param res_type: 返回形式
        :return:
        '''
        yield self.read_page()


class HttpHtmlModel(BaseHttpModel):

    def __init__(self, model_info):
        super().__init__(model_info)

    def read_page(self, page=1, pagesize=20):
        '''
        分页读取数据
        :param page:
        :param pagesize:
        :return:
        '''
        flag, err = self.connect()
        if not flag:
            return False, err
        try:
            result = self.response.text
            res_data = {
                'records': [{"html": result}],
                'total': 1
            }
            return True, gen_json_response(data=res_data)
        except Exception as e:
            return False, str(e)[:200]

    def read_batch(self):
        '''
        生成器分批读取数据
        :param res_type: 返回形式
        :return:
        '''
        yield self.read_page()





import json
from ezetl.data_models import DataModel
from ezetl.utils.common_utils import trans_rule_value, gen_json_response, parse_json
import inspect
import pandas as pd


class CCxtModel(DataModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        model_conf = self._model.get('model_conf', {})
        self.exchange_id = model_conf.get('exchange_id', '')
        self.ext_params = parse_json(self._model.get('ext_params'), {})
        self.method = model_conf.get('method', '')
        self.auth_types = model_conf.get('auth_type', '').split(',')

    def connect(self):
        '''
        连通性测试
        '''
        try:
            import ccxt
            if self.exchange_id != '':
                exchange_class = getattr(ccxt, self.exchange_id)
                self.exchange = exchange_class(self.ext_params)
                self.fetch_function = getattr(self.exchange, self.method)
                sig = inspect.signature(self.fetch_function)
                params = sig.parameters
                default_parmas = {}
                for name, param in params.items():
                    _default = param.default
                    if _default is inspect.Parameter.empty:
                        if name == 'symbol':
                            _default = 'BTC/USDT'
                        else:
                            _default = ""
                    print(name, _default)
                    default_parmas[name] = _default
                self.func_params = default_parmas
            return True, '连接成功'
        except Exception as e:
            return False, str(e)

    def get_res_fields(self):
        '''
        获取字段列表
        '''
        flag, res_data = self.gen_extract_rules()
        if not flag:
            assert ValueError(res_data)
        res_fields = []
        try:
            for column in self.df.columns:
                dic = {
                    'field_name': column,
                    'field_value': column,
                }
                res_fields.append(dic)
        except Exception as e:
            print(e)
        return res_fields

    def get_search_type_list(self):
        '''
        获取可用高级查询类型
        '''
        flag, _ = self.connect()
        if not flag:
            return []
        return [{
            'name': '查询参数',
            'value': 'query_params',
            "default": json.dumps(self.func_params, ensure_ascii=False, indent=2)
        }]

    def get_extract_rules(self):
        '''
        获取可筛选项
        :return:
        '''
        rules = [{
            'name': '等于',
            'value': 'equal'
        }, {
            'name': '不等于',
            'value': 'f_equal'
        }, {
            'name': '大于',
            'value': 'gt'
        }, {
            'name': '大于等于',
            'value': 'gte'
        }, {
            'name': '小于',
            'value': 'lt'
        }, {
            'name': '小于等于',
            'value': 'lte'
        }
        ]
        return rules

    def gen_dataframe(self, res_data):
        '''
        将目标数据转换为dataframe
        '''
        if self.method == 'fetch_ohlcv':
            data_li = []
            for ohlcv in res_data:
                dic = {
                    'exchange': self.exchange_id,
                    'time': ohlcv[0],
                    'symbol': self.func_params['symbol'],
                    'timeframe': self.func_params['timeframe'],
                    'open': ohlcv[1],
                    'high': ohlcv[2],
                    'low': ohlcv[3],
                    'close': ohlcv[4],
                    'volume': ohlcv[5]
                }
                data_li.append(dic)
            df = pd.DataFrame(data_li)
        elif self.method == 'load_markets':
            data_li = []
            for symbol, data in res_data.items():
                data['exchange'] = self.exchange_id
                data_li.append(data)
            df = pd.DataFrame(data_li)
        elif self.method in ['fetch_ticker', 'fetch_order_book', 'fetch_status']:
            res_data['exchange'] = self.exchange_id
            data_li = [res_data]
            df = pd.DataFrame(data_li)
        else:
            df = pd.DataFrame(res_data)
        return df

    def gen_extract_rules(self):
        '''
        解析筛选规则
        :return:
        '''
        # 若查询参数中有函数参数，替换函数参数
        params_rules = [i for i in self.extract_rules if i['field'] == 'search_text' and i['rule'] == 'query_params' and i['value']]
        if params_rules != []:
            self.func_params = parse_json(params_rules[0].get('value'), self.func_params)
        # 若请求参数中有函数参数，替换函数参数
        ext_query_params = [i for i in self.extract_rules if i['field'] in self.func_params and i['value']]
        if ext_query_params != []:
            for i in ext_query_params:
                self.func_params[i['field']] = i['value']
        query_params = {}
        for k in self.func_params:
            query_params[k] = trans_rule_value(self.func_params[k])
        try:
            res_data = self.fetch_function(**query_params)
            self.df = self.gen_dataframe(res_data)
        except Exception as e:
            return False, str(e)
        for i in self.extract_rules:
            field = i.get('field')
            rule = i.get('rule')
            value = i.get('value')
            value = trans_rule_value(value)
            if field and value:
                if rule in ['equal', 'eq']:
                    self.df = self.df[self.df[field] == value]
                elif rule in ['f_equal', 'neq']:
                    self.df = self.df[self.df[field] != value]
                elif rule == 'gt':
                    self.df = self.df[self.df[field] > value]
                elif rule == 'gte':
                    self.df = self.df[self.df[field] >= value]
                elif rule == 'lt':
                    self.df = self.df[self.df[field] < value]
                elif rule == 'lte':
                    self.df = self.df[self.df[field] <= value]
        self.df.fillna("", inplace=True)
        data_li = []
        for k, row in self.df.iterrows():
            data_li.append(row.to_dict())
        return True, data_li

    def read_page(self, page=1, pagesize=20):
        '''
        分页读取数据
        :param page:
        :param pagesize:
        :return:
        '''
        flag, res_data = self.connect()
        if not flag:
            return False, res_data
        flag, res_data = self.gen_extract_rules()
        if not flag:
            return False, res_data
        total = len(res_data)
        data_li = res_data[(page-1)*pagesize:page*pagesize]
        res_data = {
            'records': data_li,
            'total': total
        }
        return True, gen_json_response(data=res_data)

    def read_batch(self):
        '''
        生成器分批读取数据
        :param res_type: 返回形式
        :return:
        '''
        flag, res_data = self.connect()
        if not flag:
            return False, res_data
        flag, res_data = self.gen_extract_rules()
        if not flag:
            yield False, res_data
        total = len(res_data)
        pagesize = self._extract_info.get('batch_size', 1000)
        total_pages = total // pagesize + 1
        n = 0
        while n < total_pages:
            page = n + 1
            n += 1
            result = {
                'records': res_data[(page-1)*pagesize:page*pagesize],
                'total': total
            }
            yield True, gen_json_response(result)


from ezetl.data_models.base_db_table import BaseDBTableModel
from ezetl.data_models.base_db_sql import BaseDBSqlModel
from sqlalchemy import Column, TEXT, MetaData
from ezetl.utils.common_utils import gen_json_response
from sqlalchemy.schema import CreateTable


def gen_string_column(i):
    if 'nullable' in i.keys():
        i['nullable'] = i['nullable'] == 1
    else:
        i['nullable'] = True
    obj = Column(TEXT, nullable=i['nullable'])
    return obj


class SqlServerTableModel(BaseDBTableModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        self.column_gen_map = {
            'String': gen_string_column
        }

    def create(self, field_arr=[]):
        '''
        创建表
        '''
        if 'create' not in self.auth_types:
            return False, '无创建权限'
        table_args = self.table_args
        try:
            class Model(self.BaseModel):
                __tablename__ = self.table_name
                if table_args is not None:
                    __table_args__ = table_args

            for i in field_arr:
                if i['field_value'] == 'id':
                    continue
                if i['type'] in self.column_gen_map:
                    column = self.column_gen_map[i['type']](i)
                else:
                    column = self.genColumn(i)
                setattr(Model, i['field_value'], column)

            metadata = MetaData()
            metadata.reflect(bind=self.db_engine)
            if self.table_name not in metadata.tables.keys():
                create_sql = str(CreateTable(Model.__table__).compile(self.db_engine))
                print(create_sql)
                Model.__table__.create(self.db_engine)
            self.db_engine.dispose()
            # 重新获取模型
            self.get_table_model()
            return True, '创建成功'
        except Exception as e:
            return False, str(e)

    def read_page(self, page=1, pagesize=20):
        '''
        分页读取数据
        :param page:
        :param pagesize:
        :return:
        '''
        if self.db_model is False:
            return False, '数据库链接错误'
        flag, query = self.gen_extract_rules(self.db_model)
        if not flag:
            return False, query
        total = query.count()
        query = query.order_by(self.table.columns[0])
        query = query.offset((page - 1) * pagesize)
        query = query.limit(pagesize)
        obj_list = query.all()
        data_li = []
        for obj in obj_list:
            dic = {c.name: getattr(obj, c.name) for c in self.table.columns}
            data_li.append(dic)
        res_data = {
            'records': data_li,
            'total': total
        }
        return True, gen_json_response(res_data)

    def read_batch(self):
        '''
        生成器分批读取数据
        :return:
        '''
        if not self.db_model:
            yield False, '数据库链接错误'
        flag, query = self.gen_extract_rules(self.db_model)
        if not flag:
            yield False, query
        total = query.count()
        pagesize = self._extract_info.get('batch_size', 1000)
        total_pages = total // pagesize + 1
        n = 0
        while n < total_pages:
            page = n + 1
            n += 1
            query = query.order_by(self.table.columns[0])
            query = query.offset((page - 1) * pagesize)
            query = query.limit(pagesize)
            obj_list = query.all()
            data_li = []
            for obj in obj_list:
                dic = {c.name: getattr(obj, c.name) for c in self.table.columns}
                data_li.append(dic)
            res_data = {
                'records': data_li,
                'total': total
            }
            yield True, gen_json_response(res_data)


class SqlServerSqlModel(BaseDBSqlModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        model_conf = self._model.get('model_conf', {})
        self.sql = model_conf.get('sql', 'SELECT name FROM sys.tables')
        self.default_sql = self.sql


from setuptools import setup, find_packages

version = '1.0.23'

setup(
    name='ez-etl',
    version=version,
    description="ez-etl is an open-source Extract, Transform, load (ETL) library written in Python. Just configure a dict to read data from various data sources. Use code or built-in conversion algorithms to transform the data into the target data format， and write it to the target data source. ",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': [
        ],
    },
    author='Xu Wei',
    author_email='1013104194@qq.com',
    license='Apache License 2.0',
    install_requires=[
        "requests>=2.25.1",
        "sshtunnel",
        "python-dateutil",
        "pandas>=1.0.0",
        "openpyxl",
        "xlrd",
        "minio==7.1.1",
        "sqlalchemy>=1.2.0",
        "pymysql",
        "elasticsearch>=7.17.2",
        "redis==4.0.2",
        "kafka-python==2.0.2",
        "mysql-replication==0.27",
        "minio==7.1.1"
    ],
    extras_require={
        'all': [
            "xorbits==0.3.2",
            "pyarrow>=5.0.0",
            "akshare",
            "ccxt",
            "psycopg2-binary",
            "pymssql",
            "cx_oracle",
            "clickhouse-sqlalchemy",
            "thrift",
            "thrift_sasl",
            "pyhive",
            "py2neo",
            "prometheus-api-client",
            "mongoengine",
            "influxdb==5.3.1",
        ],
        'xorbits': [
            "xorbits==0.3.2",
            "pyarrow>=5.0.0"
        ],
    },
    keywords="etl, extract, transform, load, excel, csv, mysql, kafka, elasticsearch, neo4j, influxdb, mongodb, clickhouse",
)


from ezetl.data_models.base_db_table import BaseDBTableModel
from ezetl.data_models.base_db_sql import BaseDBSqlModel
from sqlalchemy import Column, TEXT, MetaData
from sqlalchemy.schema import CreateTable


def gen_string_column(i):
    if 'nullable' in i.keys():
        i['nullable'] = i['nullable'] == 1
    else:
        i['nullable'] = True
    obj = Column(TEXT, nullable=i['nullable'])
    return obj


class OracleTableModel(BaseDBTableModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        self.column_gen_map = {
            'String': gen_string_column
        }

    def create(self, field_arr=[]):
        '''
        创建表
        '''
        if 'create' not in self.auth_types:
            return False, '无创建权限'
        table_args = self.table_args
        try:
            class Model(self.BaseModel):
                __tablename__ = self.table_name
                if table_args is not None:
                    __table_args__ = table_args

            for i in field_arr:
                if i['field_value'] == 'id':
                    continue
                if i['type'] in self.column_gen_map:
                    column = self.column_gen_map[i['type']](i)
                else:
                    column = self.genColumn(i)
                setattr(Model, i['field_value'], column)

            metadata = MetaData()
            metadata.reflect(bind=self.db_engine)
            if self.table_name not in metadata.tables.keys():
                create_sql = str(CreateTable(Model.__table__).compile(self.db_engine))
                print(create_sql)
                Model.__table__.create(self.db_engine)
            self.db_engine.dispose()
            # 重新获取模型
            self.get_table_model()
            return True, '创建成功'
        except Exception as e:
            return False, str(e)


class OracleSqlModel(BaseDBSqlModel):

    def __init__(self, model_info):
        super().__init__(model_info)
        model_conf = self._model.get('model_conf', {})
        self.sql = model_conf.get('sql', 'SELECT table_name FROM user_tables')
        self.default_sql = self.sql